extern crate actix;
extern crate actix_web;
extern crate env_logger;
extern crate reqwest;
extern crate ron;
extern crate serde;
extern crate tokio;
#[macro_use]
extern crate tera;

mod config;
mod handlers;
mod utils;

use self::config::*;
use self::handlers::*;
use actix::System;
use actix_web::{middleware::Logger, web::resource, App, HttpServer};
use ron::de::from_str;
use std::{error::Error, fs::read_to_string, time::Duration};
use tokio::prelude::{Future, Stream};
use tokio::timer::Interval;

fn main() -> Result<(), Box<Error>> {
    let config = from_str::<Config>(&read_to_string("./endstat_conf.ron")?)?;
    let bind_addr = config.bind_address.clone();
    std::env::set_var("RUST_LOG", "actix_web=info");
    env_logger::init();

    System::run(move || {
        HttpServer::new(move || {
            let tera = compile_templates!(concat!(env!("CARGO_MANIFEST_DIR"), "/templates/**/*"));

            App::new()
                .data(State::new(&config))
                .data(tera)
                .wrap(Logger::default())
                .service(resource("/").to(index))
                .service(resource("/api").to(json_endpoint))
        })
        .bind(&bind_addr)
        .expect("Could not bind to address!")
        .start();

        tokio::spawn(
            Interval::new_interval(Duration::from_millis(5000))
                .for_each(|_| {
                    println!("Every 5 seconds");
                    Ok(())
                })
                .map_err(|_| ()),
        );
    })?;

    Ok(())
}
